#ifndef _IMAGE_H
#define _IMAGE_H

#include <cstdio>
#include <cstring>

typedef struct
{
	unsigned long fs;    	//File size
	unsigned long res;		//Reserved
	unsigned long offset;	
	unsigned long hs;		//Header Size
	unsigned long width;
	unsigned long height;
	unsigned short planes;
	unsigned short bits;
	unsigned long compress;
	unsigned long bmpdata;
	unsigned long hres;
	unsigned long vres;
	unsigned long colores;
	unsigned long icol;
}BmpHeader;

enum PixelFormat{
	FormatInvalidImage = 0,
	Format1bppIndexed, //1
	Format4bppIndexed, //2
	Format8bppIndexed, //3
	Format24bppRGB,    //4
	Format32bppARGB,   //5
};

const static int PixelFormat_bits[]     = { 0, 8, 8, 8, 24, 32};
const static int PixelFormat_filebits[] = { 0, 1, 4, 8, 24, 32};
const static int PixelFormat_colors[]   = { 0, 2, 16, 256, 0, 0};

class Image{
	
	private:		
		int _w; //width
		int _h; //height
		int _bc; //background color
		int _fc; //foreground color
		PixelFormat _pf;
		unsigned char *_d; //data
		unsigned char *_p; //palette
		//---------------------------
		int _lw; //line width
		int _size; //req no of bytes for _d;
		int _noc; //no of colors
		int _Bpp; //Bytes per pixel
		
		void _setpixelformat( const PixelFormat pf);
		unsigned char * _getDataPointer()const;
		unsigned char * _getPalettePointer()const;
		
	public:
		
		//---< CONSTRUCTORS >---
		Image( ); //default
		Image( const char *fileName);
		Image( int width, int height, PixelFormat pixelformat = Format8bppIndexed);
		//---< DESTRUCTORS >---
		~Image();
		//---< MEMBER FUNCTIONS >---
		Image& clone() const;
		Image& rawclone() const;
		void freeImage();
		bool isValid() const;
		bool isGrayImage() const;
		int getWidth() const;
		int getHeight() const;
		PixelFormat getPixelFormat() const;
		int getNoOfColors() const;
		int getPixelFormatSize() const;
		int getSize() const;
		int getBackColor( ) const;
		void setBackColor( int c);
		int getForeColor( ) const;
		void setForeColor( int c);
		int getPixel( int x, int y) const;
		void setPixel( int x, int y, int c);
		int getPalette( int i) const;
		void setPalette( int i, int c);
		int* getAllPalette( ) const;
		void setAllPalette( int *palette);
		void resetPalette();
		unsigned char* toArray( ) const;
		void fromArray( const unsigned char *data, int width, int height, PixelFormat pixelformat = Format8bppIndexed);
		void convertToRGB();
		void convertToGray();
		void convertToMono();
		bool loadBMP( const char *fname);
		void saveBMP(const char *fname) const;
		void floodfill( int x, int y, int old, int replaced);
		friend Image& crop( const Image &image, int x, int y, int w, int h);
		friend Image& hFilp( const Image &image);
		friend Image& vFilp( const Image &image);
		friend Image& extractRed( const Image &image);
		friend Image& extractGreen( const Image &image);
		friend Image& extractBlue( const Image &image);
		friend Image& mergeRGB( const Image &r, const Image &g, const Image &b);
		friend Image& convolute3x3( const Image &image, const int *filter, int divisor, int offset);
		friend Image& diluteCircle( const Image &image, float radius);
};

#endif
